
/* DDSHELL.c

   Drop and Drag Shell for Windows applications that do not implement the
   drop and drag paradigm.

   History:
   NSDavids 92-07-01
      Code started

*/

#define STRICT        /* define STRICT to provide maximum type safety
                         as defined by Microsoft */

#include <windows.h>
#include <string.h>
#include <dir.h>
#include <shellapi.h>

#define WS_WINDOWSTYLE (WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME)
#define IDM_ABOUT 1

char        szAppName [] = "Drop_and_Drag";

LPSTR       lpDDInstance;
char        szCommand [MAXPATH + 2];
char        szTitle [80];
int         initialXPos;
int         initialYPos;

long CALLBACK _export WndProc(HWND, UINT, UINT, LONG);

#pragma argsused
/**************************************************************/
int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine,     int nCmdShow)
{
MSG                msg;
WNDCLASS           wc;
HWND               hWnd;
HMENU              hMenu;

lpDDInstance = lpCmdLine;

if (!hPrevInstance)
   {
   wc.style             = CS_HREDRAW | CS_VREDRAW ;
   wc.lpfnWndProc       = WndProc;                                  
   wc.cbClsExtra        = 0;
   wc.cbWndExtra        = 0;  
   wc.hInstance         = hInstance;
   wc.hIcon             = LoadIcon (hInstance, szAppName);
   wc.hCursor           = LoadCursor (NULL, IDC_ARROW);
   wc.hbrBackground     = GetStockObject (WHITE_BRUSH);
   wc.lpszMenuName      = NULL;
   wc.lpszClassName     = szAppName;
   
   RegisterClass(&wc);
   }

hWnd = CreateWindow(szAppName, NULL, WS_WINDOWSTYLE, CW_USEDEFAULT,
                CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                NULL, NULL, hInstance, NULL);

if (!hWnd)
   return (FALSE);

hMenu = GetSystemMenu (hWnd, FALSE);
AppendMenu (hMenu, MF_SEPARATOR, 0,         NULL);
AppendMenu (hMenu, MF_STRING,    IDM_ABOUT, "About...");
   
ShowWindow(hWnd, SW_SHOWMINIMIZED);
if ((initialXPos != 0) || (initialYPos != 0))
                  MoveWindow (hWnd, initialXPos, initialYPos, 0, 0, TRUE);

SetWindowText (hWnd, (LPSTR) szTitle);

while (GetMessage(&msg, NULL, NULL, NULL))
      {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
      }

return (msg.wParam);
} /* end of WndMain */


long CALLBACK _export WndProc
              (HWND hWnd, UINT message, UINT wParam, LONG lParam)
{
int        xPos;
int        yPos;
HDROP      hDrop;
int        TotalNumberOfFiles;
char       szFileName [MAXPATH + 1];
char       szString [256];
int        i;
WORD       winExecErr;
int        len;

switch (message)
       {
       case WM_CREATE:
            if (lstrlen (lpDDInstance) == 0)
               {
               wsprintf (szString, "%s %s",
                   (LPSTR) "DDSHELL must be called with an",
                   (LPSTR) "argument to identify its instance");
               MessageBox (hWnd, szString, szAppName,
                                              MB_ICONEXCLAMATION | MB_OK);
               DestroyWindow (hWnd);
               return 0;
               }
            len = GetPrivateProfileString (lpDDInstance, "command",
                               "", szCommand, MAXPATH + 2, "ddshell.ini");
            if (len == 0)
               {
               wsprintf (szString,
                      "No command line defined for %s in DDSHELL.INI%s%s",
                      lpDDInstance, (LPSTR) "\n          or\n",
                      (LPSTR) "DDSHELL.INI not found in your WINDOWS DIR");
               MessageBox(hWnd,
                          szString, szAppName, MB_ICONEXCLAMATION | MB_OK);
               DestroyWindow (hWnd);
               return 0;
               }
            if (len > MAXPATH)
               {
               wsprintf (szString,
                  "The command line for %s \n%s %i character limit",
                  lpDDInstance, (LPSTR) "is longer than the",
                  MAXPATH);
               MessageBox(hWnd,
                         szString, szAppName, MB_ICONEXCLAMATION | MB_OK);
               DestroyWindow (hWnd);
               return 0;
               }
            GetPrivateProfileString (lpDDInstance, "title",
                                lpDDInstance, szTitle, 80, "ddshell.ini");
            initialXPos = GetPrivateProfileInt
                                  (lpDDInstance, "xPos", 0, "ddshell.ini");
            initialYPos = GetPrivateProfileInt
                                  (lpDDInstance, "yPos", 0, "ddshell.ini");
            DragAcceptFiles (hWnd , TRUE );
            return 0;

       case WM_MOVE:
            xPos = (int) LOWORD (lParam);
            yPos = (int) HIWORD (lParam);
            wsprintf (szString, "%i", xPos);
            WritePrivateProfileString
                        (lpDDInstance, "xPos", szString, "ddshell.ini");
            wsprintf (szString, "%i", yPos);
            WritePrivateProfileString
                        (lpDDInstance, "yPos", szString, "ddshell.ini");
            return 0;

       case WM_DROPFILES:
            hDrop = (HDROP) wParam;
            TotalNumberOfFiles = DragQueryFile( hDrop , 0xFFFF, NULL, 0 );
            for ( i = 0; i < TotalNumberOfFiles ; i++ )
                {
                len = DragQueryFile( hDrop , i , NULL, 0 );
                if (len > MAXPATH)
                   {
                   wsprintf (szString,
                      "The path for dropped file %i is to long", i);
                   MessageBox(hWnd,
                         szString, szAppName, MB_ICONEXCLAMATION | MB_OK);
                   }
                else
                   {
                   DragQueryFile( hDrop , i, szFileName, MAXPATH + 1 );
                   wsprintf (szString,
                        "%s %s", (LPSTR) szCommand, (LPSTR) szFileName);
                   winExecErr = WinExec (szString, SW_SHOW);
                   if (winExecErr < 32)
                      {
                      switch (winExecErr)
                             {
                             case 2:
                                  wsprintf (szString,
                                    "Invalid command path:\n%s",
                                    (LPSTR) szCommand);
                                  break;
                             default:
                                  wsprintf (szString, "%s %s %s\n%s %d",
                                    (LPSTR) "Error launching command:",
                                    (LPSTR) szCommand, (LPSTR) szFileName,
                                    (LPSTR) "error code =",
                                    winExecErr);
                             }
                      MessageBox (hWnd, szString, szAppName,
                                       MB_ICONEXCLAMATION | MB_OK);
                      } /* end if (winExecErr < 32) */
                   } /* end if (len > MAXPATH) */
                } /* end for ( i = 0; i < TotalNumberOfFiles ; i++ ) */

            DragFinish( hDrop );
            return 0;

       case WM_SYSCOMMAND:
            switch (wParam)
                   {
                   case IDM_ABOUT:
                        wsprintf (szString, "%s%s%s",
                           (LPSTR) "Drop and Drag Shell\n",
                           (LPSTR) "        version 2.0\n",
                           (LPSTR) "(c) Noah Davids 1992");
                        MessageBox(hWnd, szString, szAppName,
                                              MB_ICONEXCLAMATION | MB_OK);
                        return 0;
                   }
            break;

       case WM_CLOSE:
            DragAcceptFiles( hWnd , FALSE );
            break;

       case WM_DESTROY:
            PostQuitMessage(0);
            return 0;
       }
return DefWindowProc(hWnd, message, wParam, lParam);

} /* end of WndProc */
 
