/*****************************************************/
/* doslist.c                                         */
/* -- Enumerate all DOS boxes in the system, and     */
/*    list the current application found in each.    */
/*****************************************************/

#include <windows.h>
#include <windowsx.h>
#include "dosname.h"

#define wGetCallback    0x1684  /* Get VxD API. */
#define szApp   "DosList"   /* Class name. */
#define cbBuf   129     /* Generic buffer size. */
#define ibVM    0x00fc  /* VM global in WinOldApp. */

typedef DWORD (CALLBACK * LPFN_VXD_API)(VOID);

LRESULT CALLBACK DosListWndProc(HWND, UINT,
                                     WPARAM, LPARAM);
VOID CALLBACK    DosNameReady(DWORD, HWND);
BOOL             FRegisterApp(VOID);
VOID             GetDosName(HWND, HWND);
BOOL CALLBACK    IsWinOldApTask(HTASK);

LPFN_VXD_API     lpfn;
char             szBuf[cbBuf];
HGLOBAL          hgblLocked;

int PASCAL
WinMain(HINSTANCE hinsThis, HINSTANCE hinsPrev,
  LPSTR lszCommand, int wShowWindow)
/*****************************************************/
/* -- Entry point.                                   */
/*****************************************************/
    {
    MSG     msg;
    HWND    hwnd;

    /* First instance must register a class. */
    if (!hinsPrev)
        {
        WNDCLASS    wcs;

        wcs.style = CS_HREDRAW | CS_VREDRAW;
        wcs.lpfnWndProc = DosListWndProc;
        wcs.cbClsExtra = 0;
        wcs.cbWndExtra = 0;
        wcs.hInstance = hinsThis;
        wcs.hIcon = NULL;
        wcs.hCursor = LoadCursor(NULL, IDC_ARROW);
        wcs.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
        wcs.lpszMenuName = szApp;
        wcs.lpszClassName = szApp;
        if (!RegisterClass(&wcs))
            return FALSE;
        }

    /* Create the main window. */
    if (!(hwnd = CreateWindow(szApp, szApp,
      WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, CW_USEDEFAULT,
      GetSystemMetrics(SM_CXSCREEN) / 2,
      GetSystemMetrics(SM_CYSCREEN) / 8,
      NULL, NULL, hinsThis, NULL)))
        return FALSE;

    /* Register with the VxD. */
    if (!FRegisterApp())
        return FALSE;

    ShowWindow(hwnd, wShowWindow);

    while (GetMessage(&msg, NULL, 0, 0))
        {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
        }

    return TRUE;
    }

LRESULT CALLBACK
DosListWndProc(HWND hwnd, UINT wm, WPARAM wParam,
  LPARAM lParam)
/*****************************************************/
/* -- Main window procedure.                         */
/*****************************************************/
    {
    static  DWORD   vm;

    switch (wm)
        {
    default:
        break;

    case WM_DESTROY:
        if (hgblLocked)
            GlobalPageUnlock(hgblLocked);
        PostQuitMessage(0);
        break;

    case WM_USER:
        if (hgblLocked)
            {
            GlobalPageUnlock(hgblLocked);
            hgblLocked = NULL;
            }
        vm = lParam;
        InvalidateRect(hwnd, NULL, TRUE);
        break;

    case WM_PAINT:
        {
        char        szOut[cbBuf * 2 + 1];
        PAINTSTRUCT wps;

        BeginPaint(hwnd, &wps);
        wsprintf(szOut, "VM %lx: %s",
          vm, (LPSTR)szBuf);
        TextOut(wps.hdc, 0, 0, szOut, lstrlen(szOut));
        EndPaint(hwnd, &wps);
        }
        return TRUE;

    case WM_LBUTTONDOWN:
        SetCapture(hwnd);
        break;

    case WM_LBUTTONUP:
        if (GetCapture() == hwnd)
            {
            HWND    hwndPick;
            POINT   pt;

            ReleaseCapture();
            GetCursorPos(&pt);
            hwndPick = WindowFromPoint(pt);
            if (IsWinOldApTask(GetWindowTask(
              hwndPick)))
                GetDosName(hwnd, hwndPick);
            }
        break;
        }

    return DefWindowProc(hwnd, wm, wParam, lParam);
    }

VOID
GetDosName(HWND hwndNotify, HWND hwndDOS)
/*****************************************************/
/* -- Ask the dosname VxD for the name of the APP in */
/*    the VM given by the WinOldAp window.           */
/* -- hwndNotify    : Window to notify when ready.   */
/* -- hwndDOS       : WinOldAp window.               */
/*****************************************************/
    {
    UINT    uw1, uw2, uw3;  /* General purpose. */
    DWORD   vm;

    /* Comment in vpostd sample says we have to do */
    /* this. */
    _asm    mov hgblLocked, cs;
    GlobalPageLock(hgblLocked);

    /* Get VM handle out of WinOldApp's DS! */
    vm = *(LPDWORD)MAKELP(GetWindowInstance(hwndDOS),
      ibVM);
    uw1 = HIWORD(vm);
    uw2 = LOWORD(vm);
    uw3 = (UINT)szBuf;
    lstrcpy(szBuf, "No app detected");

    /* Call to VxD to get current DOS app name. */
    /* cx:bx : VM handle. */
    /* ds:dx : Buffer to receive DOS app name. */
    /* si    : Window to notify. */
    /* ax    : Get-path-name prot-mode API id. */
    _asm
        {
        mov     cx, uw1
        mov     bx, uw2
        mov     dx, uw3
        mov     si, hwndNotify
        mov     ax, apiDosName
        }
    (*lpfn)();

    }

VOID CALLBACK
DosNameReady(DWORD vm, HWND hwnd)
/*****************************************************/
/* -- Callback from VxD.                             */
/* -- Called when the requested VM path has been     */
/*    obtained.                                      */
/* -- Called on stack of VxD with unknown DS!        */
/* -- vm   : Handle of VM containing DOS app.        */
/* -- hwnd : Window to notify.                       */
/*****************************************************/
    {
    PostMessage(hwnd, WM_USER, 0, vm);
    }

BOOL
FRegisterApp(VOID)
/*****************************************************/
/* -- Register callback with VxD.                    */
/* -- Return false if VxD not detected or wrong      */
/*    version.                                       */
/*****************************************************/
    {
    WORD    sb, ib;

    /* Get the address of the DosName VxD's protect */
    /* mode API dispatcher. */
    _asm
        {
        xor     di, di
        mov     es, di
        mov     ax, wGetCallback
        mov     bx, wDosNameDeviceId
        int     0x2f
        mov     ib, di
        mov     sb, es
        }
    if (!(lpfn = (LPFN_VXD_API)MAKELONG(ib, sb)))
        return FALSE;

    /* Make sure expected version is installed. */
    _asm    mov     ax, apiGetVersion;
    if ((WORD)(*lpfn)() < wDosNameVersion)
        return FALSE;

    sb = HIWORD(DosNameReady);
    ib = LOWORD(DosNameReady);
    _asm
        {
        mov     dx, sb
        mov     bx, ib
        mov     ax, apiRegister
        }
    (*lpfn)();
    return TRUE;
    }
