/*****************************************************/
/* drawlist.c                                        */
/* -- DrawList() function.                           */
/*****************************************************/

#include <windows.h>
#include <windowsx.h>
#include "drawlist.h"

VOID DrawListItem(LPDRAWITEMSTRUCT, SSD *, int);

VOID
DrawList(LPDRAWITEMSTRUCT lpdis, SSD * rgssd, int cssd)
/*****************************************************/
/* -- Draw a list box string.                        */
/* -- lpdis : Windows DrawItemStruct pointer.        */
/* -- rgssd : Array of Substring definitions.        */
/* -- cssd  : Size of array.                         */
/*****************************************************/
    {
    if (lpdis->itemAction &
      (ODA_DRAWENTIRE | ODA_SELECT))
        {
        DrawListItem(lpdis, rgssd, cssd);
        if (lpdis->itemState & ODS_FOCUS)
            {
            /* Redraw erased caret. */
            DrawFocusRect(lpdis->hDC, &lpdis->rcItem);
            }
        }
    else if (lpdis->itemAction & ODA_FOCUS)
        {
        /* Toggle caret. */
        DrawFocusRect(lpdis->hDC, &lpdis->rcItem);
        }
    }

VOID
DrawListItem(LPDRAWITEMSTRUCT lpdis, SSD * rgssd,
  int cssd)
/*****************************************************/
/* -- Draw a list box string in different colors.    */
/* -- lpdis : Windows DrawItemStruct pointer.        */
/* -- rgssd : Array of SubString definitions.        */
/* -- cssd  : Size of array.                         */
/*****************************************************/
    {
    int     issd;       /* Substring index. */
    int     cchString;  /* Length of entire string. */
    HFONT   hfnt;       /* Font to use. */
    char    szBuf[100]; /* Text buffer. */

    if (lpdis->itemID == -1 ||
      !(hfnt = (HFONT)GetStockObject(ANSI_FIXED_FONT)))
        return;

    szBuf[0] = '\0';
    if ((cchString = ListBox_GetText(lpdis->hwndItem,
      lpdis->itemID, szBuf)) == LB_ERR)
        return;

    /* Erase background.  Use black if selected. */
    if (lpdis->itemState & ODS_SELECTED)
        SetBkColor(lpdis->hDC, 0x00000000);
    ExtTextOut(lpdis->hDC, 0, 0, ETO_OPAQUE,
      &lpdis->rcItem, NULL, 0, NULL);

    SetTextAlign(lpdis->hDC, TA_UPDATECP);
    MoveTo(lpdis->hDC, 0, lpdis->rcItem.top);
    SelectObject(lpdis->hDC, hfnt);

    for (issd = 0; issd < cssd; issd++)
        {
        int cchSubString;   /* Substring length. */

        SetTextColor(lpdis->hDC, rgssd[issd].clr);

        /* Determine length of this substring. */
        if (issd == cssd - 1)   
            {
            /* Last substring, draw to end. */
            cchSubString = cchString;
            }
        else
            {
            /* Interior substring, draw to next. */
            /* Don't draw off end. */
            if ((cchSubString = rgssd[issd + 1].ich) >
              cchString)
                cchSubString = cchString;
            }

        /* Subtract off base. */
        if ((cchSubString -= rgssd[issd].ich) > 0)
            TextOut(lpdis->hDC, 0, 0, 
              szBuf + rgssd[issd].ich, cchSubString);
        }
    }
