{ SWITCH.PAS                                                Donna Campanella
  Procedure to switch between Monochrome mode and EGA/VGA Color Mode for
  computers with both a color and monochrome video adapter and dual monitors.
  Does not clear the screen when switching to color mode as DOS's MODE.COM
  does.  Cursor is positioned at the row and column on the video page
  specified.  Valid modes to switch to are the 80x25 text monochrome or
  color modes indicated by the parameter MONO_MODE or COLOR_MODE.         }

unit switch;
interface
uses dos;
type
  mode = (MONO_MODE, COLOR_MODE);
const
  MONO             = 7;
  COLOR            = 3;
  VIDEO            = $10;
  SET_VIDEO_MODE   = 0;
  SET_CURSOR       = 2;
var
  regs : registers;
  CURRENT_MODE_ADDR: word absolute $0000:$0410;
  {  CURRENT_MODE_ADDR:  The two-byte word at 410h as defined in
     Norton's Programmer's Guide to the IBM PC:      (pg 52-53)
     Word that holds the equipment list that is reported
     by equipment-list service interrupt 17 (hex 11) (pg 219)
     Initial video mode:   7  6  5  4  3  2  1  0
                                 x  x
     where: 10 ==> 80 column color
            11 ==> 80 column monochrome
     Set/clear bit to switch modes.                                 }

procedure switch_mode(switch_to : mode; cursor_row, cursor_col,
                                        video_page : integer);
implementation

procedure switch_mode(switch_to : mode; cursor_row, cursor_col,
                                        video_page : integer);
var
  new_mode : integer;
  video_mode : word;
begin
  video_mode := CURRENT_MODE_ADDR;
  if switch_to = MONO_MODE then
     begin
       { set bit to switch to mono mode }
       video_mode := video_mode or $10;
       new_mode := MONO;
     end
  else
     begin
       { clear bit to switch to color mode }
       video_mode := video_mode and $EF;
       new_mode := COLOR;
     end;

  { Enable the display by updating the "equipment list"  }
  CURRENT_MODE_ADDR := video_mode;

  { Switch mode, setting high bit in AL (OR mode with    }
  { hex 80)  to avoid clearing the EGA/VGA screen.       }
  new_mode := new_mode or $80;
  regs.al := new_mode;
  regs.ah := SET_VIDEO_MODE;
  intr(VIDEO,regs);

  { Set cursor postion }
  regs.ah := SET_CURSOR;
  regs.dh := cursor_row;
  regs.bh := video_page;
  regs.dl := cursor_col;
  intr(VIDEO,regs);

end;
end.
